import { Http } from '@zeedhi/core';
import { GenericValueController } from '@/controllers/GenericValueController';
import { Utilities } from '@/util/Utilities';
import SharedInfo from '@/helper/SharedInfo';

export class CardsController {
  public DIASEST: string = '';

  public VRDESVIO: string = '';

  public VRSOLEXTRA: string = '';

  public PROCESSINGDATE: string = '';

  public PROXLIBCARD: string = '';
  
  public VRAJUSTE: string = '';
  
  public DIAAPROV: string = '';
  
  public IMPACTO: string = '';
  
  public QTDE: string = '';
  
  public QTDIASATRASO: string = '';
  
  /**
 * Populates data and applies formatting for cards content and appearance.
 *
 * This function retrieves data based on the provided components and updates card content
 * and appearance accordingly. It uses information from the components and applies formatting
 * to the displayed values. The data is fetched first with generic values, and then will be
 * populated with the DB data.
 *
 * @param {Object} components - An object containing information about the components.
 * @param {Object} components.component - The specific component information.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public getDataFillCards(components) {
    const childrenComponents = components.component.parent.children;
    const componentForProps = childrenComponents.find((child) => child.name && child.name === components.component.name);
    const props = componentForProps.props;
    const routeName = props.routeName;
    const columnName = props.columnName;
    const type = props.type;
    const sendObject = {
      filter: [],
    };
    const filter = props.filter;
    const genericValues = new GenericValueController();
    const formatting = new Utilities();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    
    if (filter) {
      sendObject.filter = formatting.formatFilter(filter);
    }
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        const data = response.data.dataset[routeName];
        const [firstRecord] = Array.isArray(data) ? data : [];
      
        this[columnName] = formatting.formatValue(type, firstRecord.VALUE);
        this.defineColor(components.component, firstRecord.VALUE);
      });
    } else {
      const object = genericValues.valueList.find((obj) => obj.columnName === columnName);
      
      if (object) {
        this[columnName] = formatting.formatValue(type, object.value);
        this.defineColor(components.component, object.value);
      } else {
        this[columnName] = 0;
        this.defineColor(components.component, 0);
      }
    }
  }
  
  /**
 * Defines the color of a component based on provided criteria and information.
 *
 * This function is responsible for determining the appropriate color for a given component based
 * on specific conditions and the values from the DB. The color calculation considers the header
 * name and their associated values to make the decision.
 *
 * @param {Object} component - The component for which the color is being defined.
 * @param {number} value - The value that influences the color determination.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public defineColor(component, value) {
    const indic = SharedInfo.getInfoIndic();
    const zdTag = component.parent.children.find((obj) => (obj.component === 'ZdTag'));
    const zdCard = component.parent;
    const lighten = new Utilities();
    const percentReduction = 18;
    let zdHeader;

    if (zdTag) {
      zdHeader = zdTag.children.find((obj) => (obj.component === 'ZdHeader'));
    } else {
      zdHeader = component.parent.children.find((obj) => (obj.component === 'ZdHeader'));
    }
    let info;

    switch (zdHeader.name) {
      case 'daysOfStockHeader':
        info = indic.find((obj) => obj.id === 1);
        zdHeader.color = this.conditionColor(value, info.limMeta, info.limMinimo, 'N');
        break;
      case 'daysOfDelayHeader':
        info = indic.find((obj) => obj.id === 3);
        zdHeader.color = this.conditionColor(value, info.limMinimo, info.limMeta, 'Y');
        break;
      case 'adjustmentValueHeader':
        info = indic.find((obj) => obj.id === 4);
        zdHeader.color = this.conditionColor(value, info.limMeta, info.limMinimo, 'Y');
        break;
      case 'extraRequestHeader':
        info = indic.find((obj) => obj.id === 5);
        zdHeader.color = this.conditionColor(value, info.limMeta, info.limMinimo, 'Y');
        break;
      case 'consumptionDiversionHeader':
        info = indic.find((obj) => obj.id === 19);
        zdHeader.color = this.conditionColor(value, info.limMeta, info.limMinimo, 'Y');
        break;
      case 'daysMenuApprovalTextHeader':
        info = indic.find((obj) => obj.id === 21);
        zdHeader.color = this.conditionColor(value, info.limMeta, info.limMinimo, 'N');
        break;
      case 'pendingReleasesHeader': zdHeader.color = '#0B5394';
        break;
      case 'processingDateHeader': zdHeader.color = '#0B5394';
        break;
      case 'nextMenuReleaseHeader': zdHeader.color = '#0B5394';
        break;
      case 'financialImpactHeader': zdHeader.color = '#0B5394';
        break;
      default: component.color = '#F3F6F4';
    }
    zdCard.color = lighten.lightenColor(zdHeader.color, percentReduction);
  }
  
  /**
 * Determine color based on value compared to specified minimum and goal values.
 *
 * This function calculates and returns a color code based on the comparison between
 * the provided value and the specified minimum and goal values. The color code is determined
 * by considering whether bigger values are better (biggerToSmaller === 'Y') or worse.
 *
 * @param {number} value - The value to be compared.
 * @param {number} minimum - The minimum threshold value.
 * @param {number} goal - The goal or optimal value.
 * @param {string} biggerToSmaller - Determines the value interpretation ('Y' for bigger values better, and vice-versa).
 * @returns {string} A color in hexadecimal.
 *
 * @access public
 * @author Grazielle Antunes
 */
  conditionColor(value: number, minimum: number, goal: number, biggerToSmaller: string): string {
    let color;
    if (biggerToSmaller === 'N') {
      color = '#F1C232';
      if (value >= goal) {
        color = '#6AA84F';
      } else if (value < minimum) {
        color = '#CC0000';
      }
    } else {
      color = '#F1C232';
      if (value < minimum) {
        color = '#6AA84F';
      } else if (value > goal) {
        color = '#CC0000';
      }
    }
    return color;
  }
  
  /**
 * Retrieves and updates table data for a specified component.
 *
 * This function is responsible for fetching and updating table data based on the provided component.
 * The data is retrieved, formatted and then applied to the component(grid) data source.
 *
 * @param {Object} component - The specific component for which table data is being managed.
 *
 * @access public
 * @author Grazielle Antunes
 */
  public tableData({ component }) {
    const routeName = component.name;
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    const formatting = new Utilities();
    const sendObject = {
      filter: [],
    };
    const filter = formatting.formatFilter(component.datasource.filter);
    
    if (filter) {
      sendObject.filter = filter;
    }
    let data;
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        component.datasource.setFilter([]);
        data = response.data.dataset[routeName].map((obj) => ({
          ...obj,
          VALUE: formatting.formatValue('float', obj.VALUE),
        }));
        component.datasource.updateData(data);
      });
    } else {
      data = genericValues.valueGridCard[component.name].map((obj) => ({
        ...obj,
        VALUE: formatting.formatValue('float', obj.VALUE),
      }));
      component.datasource.setFilter([]);
      component.datasource.updateData(data);
    }
  }
}
